
package xmlunleashed.ch10.jaxb;

import java.io.IOException;
import java.io.InputStream;
import javax.xml.bind.ConversionException;
import javax.xml.bind.Dispatcher;
import javax.xml.bind.Element;
import javax.xml.bind.InvalidAttributeException;
import javax.xml.bind.LocalValidationException;
import javax.xml.bind.MarshallableObject;
import javax.xml.bind.Marshaller;
import javax.xml.bind.MissingContentException;
import javax.xml.bind.StructureValidationException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.Validator;
import javax.xml.marshal.XMLScanner;
import javax.xml.marshal.XMLWriter;


public class Address
    extends MarshallableObject
    implements Element
{

    private String _Street;
    private String _City;
    private String _State;
    private String _PostalCode;

    public String getStreet() {
        return _Street;
    }

    public void setStreet(String _Street) {
        this._Street = _Street;
        if (_Street == null) {
            invalidate();
        }
    }

    public String getCity() {
        return _City;
    }

    public void setCity(String _City) {
        this._City = _City;
        if (_City == null) {
            invalidate();
        }
    }

    public String getState() {
        return _State;
    }

    public void setState(String _State) {
        this._State = _State;
        if (_State == null) {
            invalidate();
        }
    }

    public String getPostalCode() {
        return _PostalCode;
    }

    public void setPostalCode(String _PostalCode) {
        this._PostalCode = _PostalCode;
        if (_PostalCode == null) {
            invalidate();
        }
    }

    public void validateThis()
        throws LocalValidationException
    {
        if (_Street == null) {
            throw new MissingContentException("street");
        }
        if (_City == null) {
            throw new MissingContentException("city");
        }
        if (_State == null) {
            throw new MissingContentException("state");
        }
        if (_PostalCode == null) {
            throw new MissingContentException("postal_code");
        }
    }

    public void validate(Validator v)
        throws StructureValidationException
    {
    }

    public void marshal(Marshaller m)
        throws IOException
    {
        XMLWriter w = m.writer();
        w.start("address");
        w.leaf("street", _Street.toString());
        w.leaf("city", _City.toString());
        w.leaf("state", _State.toString());
        w.leaf("postal_code", _PostalCode.toString());
        w.end("address");
    }

    public void unmarshal(Unmarshaller u)
        throws UnmarshalException
    {
        XMLScanner xs = u.scanner();
        Validator v = u.validator();
        xs.takeStart("address");
        while (xs.atAttribute()) {
            String an = xs.takeAttributeName();
            throw new InvalidAttributeException(an);
        }
        if (xs.atStart("street")) {
            xs.takeStart("street");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Street = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("street", x);
            }
            xs.takeEnd("street");
        }
        if (xs.atStart("city")) {
            xs.takeStart("city");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _City = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("city", x);
            }
            xs.takeEnd("city");
        }
        if (xs.atStart("state")) {
            xs.takeStart("state");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _State = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("state", x);
            }
            xs.takeEnd("state");
        }
        if (xs.atStart("postal_code")) {
            xs.takeStart("postal_code");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _PostalCode = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("postal_code", x);
            }
            xs.takeEnd("postal_code");
        }
        xs.takeEnd("address");
    }

    public static Address unmarshal(InputStream in)
        throws UnmarshalException
    {
        return unmarshal(XMLScanner.open(in));
    }

    public static Address unmarshal(XMLScanner xs)
        throws UnmarshalException
    {
        return unmarshal(xs, newDispatcher());
    }

    public static Address unmarshal(XMLScanner xs, Dispatcher d)
        throws UnmarshalException
    {
        return ((Address) d.unmarshal(xs, (Address.class)));
    }

    public boolean equals(Object ob) {
        if (this == ob) {
            return true;
        }
        if (!(ob instanceof Address)) {
            return false;
        }
        Address tob = ((Address) ob);
        if (_Street!= null) {
            if (tob._Street == null) {
                return false;
            }
            if (!_Street.equals(tob._Street)) {
                return false;
            }
        } else {
            if (tob._Street!= null) {
                return false;
            }
        }
        if (_City!= null) {
            if (tob._City == null) {
                return false;
            }
            if (!_City.equals(tob._City)) {
                return false;
            }
        } else {
            if (tob._City!= null) {
                return false;
            }
        }
        if (_State!= null) {
            if (tob._State == null) {
                return false;
            }
            if (!_State.equals(tob._State)) {
                return false;
            }
        } else {
            if (tob._State!= null) {
                return false;
            }
        }
        if (_PostalCode!= null) {
            if (tob._PostalCode == null) {
                return false;
            }
            if (!_PostalCode.equals(tob._PostalCode)) {
                return false;
            }
        } else {
            if (tob._PostalCode!= null) {
                return false;
            }
        }
        return true;
    }

    public int hashCode() {
        int h = 0;
        h = ((127 *h)+((_Street!= null)?_Street.hashCode(): 0));
        h = ((127 *h)+((_City!= null)?_City.hashCode(): 0));
        h = ((127 *h)+((_State!= null)?_State.hashCode(): 0));
        h = ((127 *h)+((_PostalCode!= null)?_PostalCode.hashCode(): 0));
        return h;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("<<address");
        if (_Street!= null) {
            sb.append(" street=");
            sb.append(_Street.toString());
        }
        if (_City!= null) {
            sb.append(" city=");
            sb.append(_City.toString());
        }
        if (_State!= null) {
            sb.append(" state=");
            sb.append(_State.toString());
        }
        if (_PostalCode!= null) {
            sb.append(" postal_code=");
            sb.append(_PostalCode.toString());
        }
        sb.append(">>");
        return sb.toString();
    }

    public static Dispatcher newDispatcher() {
        Dispatcher d = new Dispatcher();
        d.register("address", (Address.class));
        d.register("contact", (Contact.class));
        d.register("rental_property", (RentalProperty.class));
        d.register("rental_property_list", (RentalPropertyList.class));
        d.freezeElementNameMap();
        return d;
    }

}
