
package xmlunleashed.ch10.jaxb;

import java.io.IOException;
import java.io.InputStream;
import javax.xml.bind.ConversionException;
import javax.xml.bind.Dispatcher;
import javax.xml.bind.Element;
import javax.xml.bind.InvalidAttributeException;
import javax.xml.bind.LocalValidationException;
import javax.xml.bind.MarshallableObject;
import javax.xml.bind.Marshaller;
import javax.xml.bind.MissingContentException;
import javax.xml.bind.StructureValidationException;
import javax.xml.bind.UnmarshalException;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.Validator;
import javax.xml.marshal.XMLScanner;
import javax.xml.marshal.XMLWriter;


public class Contact
    extends MarshallableObject
    implements Element
{

    private String _Phone;
    private String _Fax;

    public String getPhone() {
        return _Phone;
    }

    public void setPhone(String _Phone) {
        this._Phone = _Phone;
        if (_Phone == null) {
            invalidate();
        }
    }

    public String getFax() {
        return _Fax;
    }

    public void setFax(String _Fax) {
        this._Fax = _Fax;
        if (_Fax == null) {
            invalidate();
        }
    }

    public void validateThis()
        throws LocalValidationException
    {
        if (_Phone == null) {
            throw new MissingContentException("phone");
        }
        if (_Fax == null) {
            throw new MissingContentException("fax");
        }
    }

    public void validate(Validator v)
        throws StructureValidationException
    {
    }

    public void marshal(Marshaller m)
        throws IOException
    {
        XMLWriter w = m.writer();
        w.start("contact");
        w.leaf("phone", _Phone.toString());
        w.leaf("fax", _Fax.toString());
        w.end("contact");
    }

    public void unmarshal(Unmarshaller u)
        throws UnmarshalException
    {
        XMLScanner xs = u.scanner();
        Validator v = u.validator();
        xs.takeStart("contact");
        while (xs.atAttribute()) {
            String an = xs.takeAttributeName();
            throw new InvalidAttributeException(an);
        }
        if (xs.atStart("phone")) {
            xs.takeStart("phone");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Phone = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("phone", x);
            }
            xs.takeEnd("phone");
        }
        if (xs.atStart("fax")) {
            xs.takeStart("fax");
            String s;
            if (xs.atChars(XMLScanner.WS_COLLAPSE)) {
                s = xs.takeChars(XMLScanner.WS_COLLAPSE);
            } else {
                s = "";
            }
            try {
                _Fax = String.valueOf(s);
            } catch (Exception x) {
                throw new ConversionException("fax", x);
            }
            xs.takeEnd("fax");
        }
        xs.takeEnd("contact");
    }

    public static Contact unmarshal(InputStream in)
        throws UnmarshalException
    {
        return unmarshal(XMLScanner.open(in));
    }

    public static Contact unmarshal(XMLScanner xs)
        throws UnmarshalException
    {
        return unmarshal(xs, newDispatcher());
    }

    public static Contact unmarshal(XMLScanner xs, Dispatcher d)
        throws UnmarshalException
    {
        return ((Contact) d.unmarshal(xs, (Contact.class)));
    }

    public boolean equals(Object ob) {
        if (this == ob) {
            return true;
        }
        if (!(ob instanceof Contact)) {
            return false;
        }
        Contact tob = ((Contact) ob);
        if (_Phone!= null) {
            if (tob._Phone == null) {
                return false;
            }
            if (!_Phone.equals(tob._Phone)) {
                return false;
            }
        } else {
            if (tob._Phone!= null) {
                return false;
            }
        }
        if (_Fax!= null) {
            if (tob._Fax == null) {
                return false;
            }
            if (!_Fax.equals(tob._Fax)) {
                return false;
            }
        } else {
            if (tob._Fax!= null) {
                return false;
            }
        }
        return true;
    }

    public int hashCode() {
        int h = 0;
        h = ((127 *h)+((_Phone!= null)?_Phone.hashCode(): 0));
        h = ((127 *h)+((_Fax!= null)?_Fax.hashCode(): 0));
        return h;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("<<contact");
        if (_Phone!= null) {
            sb.append(" phone=");
            sb.append(_Phone.toString());
        }
        if (_Fax!= null) {
            sb.append(" fax=");
            sb.append(_Fax.toString());
        }
        sb.append(">>");
        return sb.toString();
    }

    public static Dispatcher newDispatcher() {
        return Address.newDispatcher();
    }

}
